@@ -12,7 +12,7 @@ from ..hash import hash256, sha256
 from ..keys import PrivateKey
 from ..script.script import Script
 from ..script.type import ScriptType, P2pkhScriptType, OpReturnScriptType, UnknownScriptType
-from ..service.provider import Provider
+from ..service.provider import Provider, BroadcastResult
 from ..service.service import Service
 from ..utils import unsigned_to_varint
 
@@ -147,14 +147,18 @@ class Transaction:
 
     def __init__(self, tx_inputs: Optional[List[TxInput]] = None, tx_outputs: Optional[List[TxOutput]] = None,
                  version: int = TRANSACTION_VERSION, locktime: int = TRANSACTION_LOCKTIME, fee_rate: Optional[float] = None,
-                 chain: Chain = Chain.MAIN, provider: Provider = None, **kwargs):
+                 chain: Optional[Chain] = None, provider: Optional[Provider] = None, **kwargs):
         self.tx_inputs: List[TxInput] = tx_inputs or []
         self.tx_outputs: List[TxOutput] = tx_outputs or []
         self.version: int = version
         self.locktime: int = locktime
         self.fee_rate: float = fee_rate if fee_rate is not None else TRANSACTION_FEE_RATE
+
         self.chain: Chain = chain
         self.provider: Provider = provider
+        if self.provider:
+            self.chain = self.provider.chain
+
         self.kwargs: Dict[str, Any] = dict(**kwargs) or {}
 
     def serialize(self) -> bytes:
@@ -314,13 +318,13 @@ class Transaction:
 
     def sign(self, bypass: bool = True, **kwargs) -> 'Transaction':  # pragma: no cover
         """
-        :bypass: if True then ONLY sign inputs which unlocking script is empty, otherwise sign all the inputs
+        :bypass: if True then ONLY sign inputs which unlocking script is None, otherwise sign all the inputs
         sign all inputs according to their script type
         """
         digests = self.digests()
         for i in range(len(self.tx_inputs)):
             tx_input = self.tx_inputs[i]
-            if not tx_input.unlocking_script or not bypass:
+            if tx_input.unlocking_script is None or not bypass:
                 signatures: List[bytes] = [private_key.sign(digests[i]) for private_key in tx_input.private_keys]
                 payload = {'signatures': signatures, 'private_keys': tx_input.private_keys, 'sighash': tx_input.sighash}
                 tx_input.unlocking_script = tx_input.script_type.unlocking(**payload, **{**self.kwargs, **kwargs})
@@ -352,9 +356,13 @@ class Transaction:
         """
         estimated_length = 4 + len(unsigned_to_varint(len(self.tx_inputs))) + len(unsigned_to_varint(len(self.tx_outputs))) + 4
         for tx_input in self.tx_inputs:
-            if not tx_input.private_keys:
-                raise ValueError(f"can't estimate byte length for {tx_input} without private keys")
-            estimated_length += 41 + tx_input.script_type.estimated_unlocking_byte_length(private_keys=tx_input.private_keys, **{**self.kwargs, **kwargs})
+            if tx_input.unlocking_script is not None:
+                # unlocking script already set
+                estimated_length += len(tx_input.serialize())
+            else:
+                if not tx_input.private_keys:
+                    raise ValueError(f"can't estimate byte length for {tx_input} without private keys")
+                estimated_length += 41 + tx_input.script_type.estimated_unlocking_byte_length(private_keys=tx_input.private_keys, **{**self.kwargs, **kwargs})
         for tx_output in self.tx_outputs:
             estimated_length += 8 + len(tx_output.locking_script.byte_length_varint()) + tx_output.locking_script.byte_length()
         return estimated_length
@@ -378,7 +386,7 @@ class Transaction:
             if not change_address:
                 for tx_input in self.tx_inputs:  # pragma: no cover
                     if tx_input.script_type == P2pkhScriptType():
-                        change_output = TxOutput(out=tx_input.locking_script, satoshi=fee_overpaid)
+                        change_output = TxOutput(out=tx_input.locking_script, satoshi=fee_overpaid, script_type=P2pkhScriptType())
                         break
             else:
                 change_output = TxOutput(out=change_address, satoshi=fee_overpaid)
@@ -386,12 +394,32 @@ class Transaction:
             self.add_output(change_output)
         return self
 
-    def broadcast(self) -> Optional[str]:  # pragma: no cover
+    def broadcast(self) -> BroadcastResult:  # pragma: no cover
         fee_expected = math.ceil(self.fee_rate * self.byte_length())
         if self.fee() < fee_expected:
             raise InsufficientFunds(f'require {self.satoshi_total_out() + fee_expected} satoshi but only {self.satoshi_total_in()}')
         return Service(self.chain, self.provider).broadcast(self.hex())
 
+    def to_unspent(self, vout: int, **kwargs) -> Optional[Unspent]:
+        assert 0 <= vout < len(self.tx_outputs), 'vout out of range'
+        out = self.tx_outputs[vout]
+        if out.script_type in [OpReturnScriptType()]:
+            return None
+        return Unspent(txid=self.txid(), vout=vout, satoshi=out.satoshi, script_type=out.script_type, locking_script=out.locking_script, **kwargs)
+
+    def to_unspents(self, vouts: Optional[List[int]] = None, args: Optional[List[Dict]] = None) -> List[Unspent]:
+        """
+        parse all the outputs to unspents if vouts is None or empty, OP_RETURN outputs will be omitted
+        """
+        vouts = vouts or range(len(self.tx_outputs))
+        unspents = []
+        for i in range(len(vouts)):
+            arg = args[i] if args and 0 <= i < len(args) else {}
+            unspent = self.to_unspent(vouts[i], **arg)
+            if unspent:
+                unspents.append(unspent)
+        return unspents
+
     @classmethod
     def from_hex(cls, stream: Union[str, bytes, TransactionBytesIO]) -> Optional['Transaction']:
         with suppress(Exception):
